@@ -3,11 +3,11 @@
 namespace App\Classes\LDAP\Attribute;
 
 use Illuminate\Contracts\View\View;
-use Illuminate\Support\Arr;
 use Illuminate\Support\Collection;
 
 use App\Classes\LDAP\Attribute;
 use App\Classes\Template;
+use App\Ldap\Entry;
 use App\Traits\MD5Updates;
 
 /**
@@ -17,6 +17,8 @@ final class Password extends Attribute
 {
 	use MD5Updates;
 
+	public const obfuscate = '****************';
+
 	protected(set) bool $no_attr_tags = TRUE;
 	protected(set) int $max_values_count = 1;
 
@@ -52,7 +54,7 @@ final class Password extends Attribute
 		$m = [];
 		preg_match('/^{([A-Z0-9]+)}(.*)$/',$password,$m);
 
-		$hash = Arr::get($m,1,'*clear*');
+		$hash = \Arr::get($m,1,'*clear*');
 
 		if (($potential=static::helpers()->filter(fn($hasher)=>str_starts_with($hasher::key,$hash)))->count() > 1) {
 			foreach ($potential as $item) {
@@ -80,15 +82,19 @@ final class Password extends Attribute
 		return ($helpers=static::helpers())->has($id) ? new ($helpers->get($id)) : NULL;
 	}
 
-	public function render(bool $edit=FALSE,bool $old=FALSE,bool $new=FALSE,bool $updated=FALSE,?Template $template=NULL): View
+	public function render(string $attrtag,int $index,bool $edit=FALSE,bool $editable=FALSE,bool $new=FALSE,bool $updated=FALSE,?Template $template=NULL): View
 	{
-		return view('components.attribute.password')
+		return view('components.attribute.value.password')
 			->with('o',$this)
+			->with('dotkey',$dotkey=$this->dotkey($attrtag,$index))
+			->with('value',$this->values->dot()->get($dotkey))
 			->with('edit',$edit)
-			->with('old',$old)
+			->with('editable',$editable)
 			->with('new',$new)
-			->with('template',$template)
+			->with('attrtag',$attrtag)
+			->with('index',$index)
 			->with('updated',$updated)
+			->with('template',$template)
 			->with('helpers',static::helpers()->map(fn($item,$key)=>['id'=>$key,'value'=>$key])->sort());
 	}
 
@@ -97,8 +103,8 @@ final class Password extends Attribute
 		$pw = parent::render_item_old($dotkey);
 
 		return $pw
-			? (((($x=$this->hash($pw)) && ($x::id() === '*clear*')) ? sprintf('{%s}',$x::shortid()) : '')
-				.str_repeat('*',16))
+			? (((($x=$this->hash($pw)) && ($x->id() !== '*clear*')) ? sprintf('{%s}',$x->shortid()) : '')
+				.self::obfuscate)
 			: NULL;
 	}
 
@@ -107,8 +113,26 @@ final class Password extends Attribute
 		$pw = parent::render_item_new($dotkey);
 
 		return $pw
-			? (((($x=$this->hash($pw)) && ($x::id() !== '*clear*')) ? sprintf('{%s}',$x::shortid()) : '')
-				.str_repeat('*',16))
+			? ((($x=$this->hash($pw)) && ($x->id() === '*clear*')) ? sprintf('{%s}%s',$x->shortid(),self::obfuscate) : $pw)
 			: NULL;
 	}
+
+	protected function setValuesHelper(Collection $values): Collection
+	{
+		$processed = collect();
+
+		// If the attr tags are the same value as the md5 tag, then nothing has changed
+		foreach ($this->keys as $key) {
+			foreach ($values->get($key,[]) as $index => $value) {
+				$helper = $values->dot()->get($key.Entry::TAG_HELPER.'.'.$index);
+
+				$processed->put($key.'.'.$index,
+					($value && $helper && ($helper !== self::hash($value)->id()))
+						? self::hash_id($helper)->encode($value)
+						: $value);
+			}
+		}
+
+		return $processed->undot();
+	}
 }
\ No newline at end of file
